@@ -175,27 +175,57 @@ cat > $CONF_FILE <<EOF
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;\\n\\n"))
 
 (setq load-path (quote $LOAD_PATH))
-(require 'package)
-;; User may be using a non standard \`package-user-dir'.
-;; Modify \`package-directory-list' instead of \`package-user-dir'
-;; in case the user starts Helm from a non-ELPA installation.
-(unless (file-equal-p package-user-dir (locate-user-emacs-file "elpa"))
-  (add-to-list 'package-directory-list (directory-file-name
-                                        (file-name-directory
-                                         (directory-file-name default-directory)))))
 
-(let* ((str-lst "$LOAD_PACKAGES")
-       (load-packages (and str-lst
-                           (not (string= str-lst ""))
-                           (split-string str-lst ","))))
-  (setq package-load-list
-        (if (equal load-packages '("all"))
-            '(all)
-          (append '((helm-core t) (helm t) (async t) (popup t))
-                  (mapcar (lambda (p) (list (intern p) t)) load-packages)))))
+(defvar default-package-manager nil)
+(defvar bootstrap-version)
+(let* ((packages "$LOAD_PACKAGES")
+       (pkg-list (and packages
+                      (not (equal packages ""))
+                      (split-string packages ",")))
+       (straight-path (expand-file-name "straight/build/" user-emacs-directory))
+       (async-path (expand-file-name "straight/build/async" user-emacs-directory))
+       (bootstrap-file
+        (expand-file-name "straight/repos/straight.el/bootstrap.el" user-emacs-directory))
+       (bootstrap-version 5))
+  (when (file-exists-p bootstrap-file)
+    (setq default-package-manager 'straight)
+    (load bootstrap-file nil 'nomessage)
+    (add-to-list 'load-path async-path)
+    (when pkg-list
+      (dolist (pkg pkg-list)
+        (let* ((pkg-path (expand-file-name pkg straight-path))
+               (autoload-file (expand-file-name
+                               (format "%s-autoloads.el" pkg)
+                               pkg-path)))
+          (add-to-list 'load-path pkg-path)
+          (if (file-exists-p autoload-file)
+              (load autoload-file nil 'nomessage)
+            (straight-use-package (intern pkg))))))))
+
+(unless (eq default-package-manager 'straight)
+  (require 'package)
+  ;; User may be using a non standard \`package-user-dir'.
+  ;; Modify \`package-directory-list' instead of \`package-user-dir'
+  ;; in case the user starts Helm from a non-ELPA installation.
+  (unless (file-equal-p package-user-dir (locate-user-emacs-file "elpa"))
+    (add-to-list 'package-directory-list (directory-file-name
+                                          (file-name-directory
+                                           (directory-file-name default-directory)))))
+
+  (let* ((str-lst "$LOAD_PACKAGES")
+         (load-packages (and str-lst
+                             (not (string= str-lst ""))
+                             (split-string str-lst ","))))
+    (setq package-load-list
+          (if (equal load-packages '("all"))
+              '(all)
+            (append '((helm-core t) (helm t) (async t) (popup t))
+                    (mapcar (lambda (p) (list (intern p) t)) load-packages)))))
+
+  (package-initialize))
 
-(package-initialize)
 (add-to-list 'load-path (file-name-directory (file-truename "$0")))
+
 (unless (> $TOOLBARS 0)
    (setq default-frame-alist '((vertical-scroll-bars . nil)
                                (tool-bar-lines . 0)
