@@ -26,8 +26,6 @@
 (require 'helm-help)
 (require 'helm-occur)
 
-(declare-function ido-make-buffer-list "ido" (default))
-(declare-function ido-add-virtual-buffers-to-list "ido")
 (declare-function helm-comp-read "helm-mode")
 (declare-function helm-browse-project "helm-files")
 
@@ -62,18 +60,18 @@ These buffers will be displayed even if they match one of
 
 (defcustom helm-buffer-max-length 20
   "Max length of buffer names before truncate.
-When disabled (nil) use the longest buffer-name length found."
+When disabled (nil) use the longest `buffer-name' length found."
   :group 'helm-buffers
   :type  '(choice (const :tag "Disabled" nil)
            (integer :tag "Length before truncate")))
 
 (defcustom helm-buffer-details-flag t
-  "Always show details in buffer list when non--nil."
+  "Always show details in buffer list when non-nil."
   :group 'helm-buffers
   :type 'boolean)
 
 (defcustom helm-buffers-fuzzy-matching nil
-  "Fuzzy matching buffer names when non--nil.
+  "Fuzzy matching buffer names when non-nil.
 Only buffer names are fuzzy matched when this is enabled,
 `major-mode' matching is not affected by this."
   :group 'helm-buffers
@@ -85,7 +83,7 @@ Only buffer names are fuzzy matched when this is enabled,
   :type 'boolean)
 
 (defcustom helm-buffers-truncate-lines t
-  "Truncate lines in `helm-buffers-list' when non--nil."
+  "Truncate lines in `helm-buffers-list' when non-nil."
   :group 'helm-buffers
   :type 'boolean)
 
@@ -99,8 +97,8 @@ Only buffer names are fuzzy matched when this is enabled,
                                        helm-source-buffer-not-found)
   "Default sources list used in `helm-mini'.
 
-When adding a source here it is up to you to ensure the library of
-this source is accessible and properly loaded."
+When adding a source here it is up to you to ensure the library
+of this source is accessible and properly loaded."
   :group 'helm-buffers
   :type '(repeat (choice symbol)))
 
@@ -117,21 +115,41 @@ this source is accessible and properly loaded."
 (defcustom helm-buffer--pretty-names '((dired-mode . "Dired")
                                        (lisp-interaction-mode . "Lisp Inter"))
   "An alist specifying pretty names for modes.
-Most of the time buffer's `mode-name' is a string so no need to add it
-here as there is no need to compute it, but sometimes it may be a
-mode-line specification which may be costly to compute, in this case
-add here the pretty name as a string to avoid this costly computation.
-Also if some pretty names are too long you can add your own
-abbreviation here."
+Most of the time buffer's `mode-name' is a string so no need to
+add it here as there is no need to compute it, but sometimes it
+may be a mode-line specification which may be costly to compute,
+in this case add here the pretty name as a string to avoid this
+costly computation.  Also if some pretty names are too long you
+can add your own abbreviation here."
   :type '(alist :key-type symbol :value-type string)
   :group 'helm-buffers)
 
 (defcustom helm-buffers-maybe-switch-to-tab nil
   "Switch to buffer in its tab when non nil.
-This have no effect when `tab-bar-mode' is not available."
+This has no effect when `tab-bar-mode' is not available."
   :group 'helm-buffers
   :type 'boolean)
 
+(defcustom helm-buffer-list-reorder-fn #'helm-buffers-reorder-buffer-list
+  "A function in charge of ordering the initial buffer list.
+It takes two arguments VISIBLES buffers and OTHERS buffers.
+Arg VISIBLES handles the buffers visibles in this frame.
+Arg OTHERS handles all the other buffers.
+You can write a function that reorder VISIBLES and OTHERS as you
+want.
+Default function returns OTHERS buffers on top and VISIBLES
+buffer at the end.  See `helm-buffers-reorder-buffer-list'."
+  :group 'helm-buffers
+  :type 'function)
+
+(defcustom helm-buffers-sort-fn helm-fuzzy-sort-fn
+  "The sort function to use in `helm-buffers-list'.
+
+Default to `helm-fuzzy-sort-fn' you can use
+`helm-fuzzy-matching-sort-fn-preserve-ties-order' as alternative if
+you want to keep the recentest order when narrowing candidates."
+  :type 'function
+  :group 'helm-buffers)
 
 ;;; Faces
 ;;
@@ -143,57 +161,69 @@ This have no effect when `tab-bar-mode' is not available."
   :group 'helm-faces)
 
 (defface helm-buffer-saved-out
-    '((t (:foreground "red" :background "black")))
+  `((t ,@(and (>= emacs-major-version 27) '(:extend t))
+       :foreground "red" :background "black"))
   "Face used for buffer files modified outside of emacs."
   :group 'helm-buffers-faces)
 
 (defface helm-buffer-not-saved
-    '((t (:foreground "Indianred2")))
+  `((t ,@(and (>= emacs-major-version 27) '(:extend t))
+       :foreground "Indianred2"))
   "Face used for buffer files not already saved on disk."
   :group 'helm-buffers-faces)
 
 (defface helm-buffer-modified
-    '((t :inherit font-lock-comment-face))
+  `((t ,@(and (>= emacs-major-version 27) '(:extend t))
+       :inherit font-lock-comment-face))
   "Face used for modified buffers."
   :group 'helm-buffers-faces)
 
 (defface helm-buffer-size
-    '((((background dark)) :foreground "RosyBrown")
-      (((background light)) :foreground "SlateGray"))
+  `((((background dark))
+     ,@(and (>= emacs-major-version 27) '(:extend t))
+     :foreground "RosyBrown")
+    (((background light))
+     ,@(and (>= emacs-major-version 27) '(:extend t))
+     :foreground "SlateGray"))
   "Face used for buffer size."
   :group 'helm-buffers-faces)
 
 (defface helm-buffer-process
-    '((t (:foreground "Sienna3")))
+  `((t ,@(and (>= emacs-major-version 27) '(:extend t))
+       :foreground "Sienna3"))
   "Face used for process status in buffer."
   :group 'helm-buffers-faces)
 
 (defface helm-buffer-directory
-    '((t (:foreground "DarkRed" :background "LightGray")))
+  `((t ,@(and (>= emacs-major-version 27) '(:extend t))
+       :foreground "DarkRed" :background "LightGray"))
   "Face used for directories in `helm-buffers-list'."
   :group 'helm-buffers-faces)
 
 (defface helm-buffer-file
-    '((t :inherit font-lock-builtin-face))
+  `((t ,@(and (>= emacs-major-version 27) '(:extend t))
+       :inherit font-lock-builtin-face))
   "Face for buffer file names in `helm-buffers-list'."
   :group 'helm-buffers-faces)
 
 (defface helm-buffer-archive
-    '((t (:foreground "Gold")))
+  `((t ,@(and (>= emacs-major-version 27) '(:extend t))
+       :foreground "Gold"))
   "Face for archive file names in `helm-buffers-list'."
   :group 'helm-buffers-faces)
 
 (defface helm-non-file-buffer
-    '((t (:inherit italic)))
+  `((t ,@(and (>= emacs-major-version 27) '(:extend t))
+       :inherit italic))
   "Face used for non-file buffers in `helm-buffers-list'."
   :group 'helm-buffers-faces)
 
 (defvar helm-buffers-tick-counter nil
   "Allows recording local changes to a non-file buffer.
-Typical usage of this var is for modes that want to see if
-their buffers have changed since last visit.
-Such programs may want to record tick counter after visiting their
-buffers like this:
+Typical usage of this var is for modes that want to see if their
+buffers have changed since last visit.
+Such programs may want to record tick counter after visiting
+their buffers like this:
 
     (setq helm-buffers-tick-counter (buffer-modified-tick))
 
@@ -235,17 +265,6 @@ Note that this variable is buffer-local.")
     map)
   "Keymap for buffer sources in helm.")
 
-(defvar helm-buffers-ido-virtual-map
-  (let ((map (make-sparse-keymap)))
-    (set-keymap-parent map helm-map)
-    (define-key map (kbd "C-c o")   'helm-ff-run-switch-other-window)
-    (define-key map (kbd "C-c C-o") 'helm-ff-run-switch-other-frame)
-    (define-key map (kbd "M-g s")   'helm-ff-run-grep)
-    (define-key map (kbd "M-g z")   'helm-ff-run-zgrep)
-    (define-key map (kbd "M-D")     'helm-ff-run-delete-file)
-    (define-key map (kbd "C-c C-x") 'helm-ff-run-open-file-externally)
-    map))
-
 
 (defvar helm-buffer-max-len-mode nil)
 (defvar helm-buffers-in-project-p nil)
@@ -353,48 +372,36 @@ Note that this variable is buffer-local.")
             #'helm-buffers-create-new-buffer-of)
    :keymap helm-buffer-not-found-map))
 
-(defvar ido-temp-list)
-(defvar ido-ignored-list)
-(defvar ido-process-ignore-lists)
-(defvar ido-use-virtual-buffers)
-(defvar ido-virtual-buffers)
-
-(defvar helm-source-ido-virtual-buffers
-  (helm-build-sync-source "Ido virtual buffers"
-    :candidates (lambda ()
-                  (let (ido-temp-list
-                        ido-ignored-list
-                        (ido-process-ignore-lists t))
-                    (when ido-use-virtual-buffers
-                      (ido-add-virtual-buffers-to-list)
-                      ido-virtual-buffers)))
-    :fuzzy-match helm-buffers-fuzzy-matching
-    :keymap helm-buffers-ido-virtual-map
-    :help-message 'helm-buffers-ido-virtual-help-message
-    :action '(("Find file" . helm-find-many-files)
-              ("Find file other window" . find-file-other-window)
-              ("Find file other frame" . find-file-other-frame)
-              ("Find file as root" . helm-find-file-as-root)
-              ("Grep File(s) `C-u recurse'" . helm-find-files-grep)
-              ("Zgrep File(s) `C-u Recurse'" . helm-ff-zgrep)
-              ("View file" . view-file)
-              ("Delete file(s)" . helm-delete-marked-files)
-              ("Open file externally (C-u to choose)"
-               . helm-open-file-externally))))
-
 
-(defvar ido-use-virtual-buffers)
-(defvar ido-ignore-buffers)
+(defun helm-buffers-get-visible-buffers ()
+  "Returns buffers visibles on current frame."
+  (let (result)
+    (walk-windows
+     (lambda (x)
+       (push (buffer-name (window-buffer x)) result))
+     nil 'visible)
+    result))
+
+(defun helm-buffer-list-1 (&optional visibles)
+  (cl-loop for b in (buffer-list)
+           for bn = (buffer-name b)
+           unless (member bn visibles)
+           collect bn))
+
+(defun helm-buffers-reorder-buffer-list (visibles others)
+  "Default function to reorder buffer-list.
+Arg VISIBLES handles the buffers visibles in this frame.
+Arg OTHERS handles all the other buffers.
+This function returns OTHERS buffers on top and VISIBLES buffer
+at the end."
+  (nconc others visibles))
+
 (defun helm-buffer-list ()
   "Return the current list of buffers.
-Currently visible buffers are put at the end of the list.
-See `ido-make-buffer-list' for more infos."
-  (require 'ido)
-  (let ((ido-process-ignore-lists t)
-        ido-ignored-list
-        ido-ignore-buffers
-        ido-use-virtual-buffers)
-    (ido-make-buffer-list nil)))
+The list is reordered with `helm-buffer-list-reorder-fn'."
+  (let* ((visibles (helm-buffers-get-visible-buffers))
+         (others   (helm-buffer-list-1 visibles)))
+    (funcall helm-buffer-list-reorder-fn visibles others)))
 
 (defun helm-buffer-size (buffer)
   "Return size of BUFFER."
@@ -504,7 +511,8 @@ See `ido-make-buffer-list' for more infos."
 
 (defun helm-highlight-buffers (buffers _source)
   "Transformer function to highlight BUFFERS list.
-Should be called after others transformers i.e (boring buffers)."
+Should be called after others transformers i.e. (boring
+buffers)."
   (cl-loop for i in buffers
            for (name size mode meta) = (if helm-buffer-details-flag
                                            (helm-buffer--details i 'details)
@@ -585,12 +593,8 @@ Should be called after others transformers i.e (boring buffers)."
 (defun helm-buffers-sort-transformer (candidates source)
   (if (string= helm-pattern "")
       candidates
-      (if helm-buffers-fuzzy-matching
-          (let ((helm-pattern (helm-buffers--pattern-sans-filters)))
-            (funcall helm-fuzzy-sort-fn candidates source))
-          (sort candidates
-                (lambda (s1 s2)
-                  (< (string-width s1) (string-width s2)))))))
+    (let ((helm-pattern (helm-buffers--pattern-sans-filters)))
+      (funcall helm-buffers-sort-fn candidates source))))
 
 (defun helm-buffers-mark-similar-buffers-1 (&optional type)
   (with-helm-window
@@ -622,7 +626,7 @@ Should be called after others transformers i.e (boring buffers)."
 
 (defun helm-buffers-mark-similar-buffers ()
     "Mark All buffers that have same property `type' than current.
-i.e same color."
+I.e. same color."
   (interactive)
   (with-helm-alive-p
     (let ((marked (helm-marked-candidates)))
@@ -830,7 +834,7 @@ If REGEXP-FLAG is given use `query-replace-regexp'."
   (helm-buffer-save-and-update nil))
 
 (defun helm-buffer-run-save-some-buffers ()
-  "Save unsaved file buffers without quitting helm."
+  "Save unsaved file buffers without quitting Helm."
   (interactive)
   (with-helm-alive-p
     (helm-attrset 'save-some-action '(helm-buffer-save-some-buffers . never-split))
@@ -838,7 +842,7 @@ If REGEXP-FLAG is given use `query-replace-regexp'."
 (put 'helm-buffer-run-save-some-buffers 'helm-only t)
 
 (defun helm-buffer-save-persistent ()
-  "Save buffer without quitting helm."
+  "Save buffer without quitting Helm."
   (interactive)
   (with-helm-alive-p
     (helm-attrset 'save-action '(helm-buffer-save-and-update . never-split))
@@ -857,7 +861,7 @@ If REGEXP-FLAG is given use `query-replace-regexp'."
 (put 'helm-buffer-run-rename-buffer 'helm-only t)
 
 (defun helm-buffer-run-kill-persistent ()
-  "Kill buffer without quitting helm."
+  "Kill buffer without quitting Helm."
   (interactive)
   (with-helm-alive-p
     (helm-attrset 'kill-action '(helm-buffers-persistent-kill . never-split))
@@ -933,8 +937,9 @@ If REGEXP-FLAG is given use `query-replace-regexp'."
 (defun helm-buffer-switch-buffers (_candidate)
   "Switch to buffer candidates and replace current buffer.
 
-If more than one buffer marked switch to these buffers in separate windows.
-If a prefix arg is given split windows vertically."
+If more than one buffer marked switch to these buffers in
+separate windows.  If a prefix arg is given split windows
+vertically."
   (let ((buffers (helm-marked-candidates)))
     (helm-window-show-buffers buffers)))
 
@@ -1088,7 +1093,7 @@ Can be used by any source that list buffers."
 (put 'helm-buffers-toggle-show-hidden-buffers 'helm-only t)
 
 (defun helm-buffers-browse-project (buf)
-  "Browse project from buffer."
+  "Browse project from buffer BUF."
   (with-current-buffer buf
     (helm-browse-project helm-current-prefix-arg)))
 
@@ -1122,7 +1127,6 @@ displayed with the `file-name-shadow' face if available."
     (setq helm-source-buffers-list
           (helm-make-source "Buffers" 'helm-source-buffers)))
   (helm :sources '(helm-source-buffers-list
-                   helm-source-ido-virtual-buffers
                    helm-source-buffer-not-found)
         :buffer "*helm buffers*"
         :keymap helm-buffer-map
