@@ -29,12 +29,12 @@
   :group 'helm)
 
 (defcustom helm-M-x-always-save-history nil
-  "`helm-M-x' Save command in `extended-command-history' even when it fail."
+  "`helm-M-x' save command in `extended-command-history' even when it fails."
   :group 'helm-command
   :type  'boolean)
 
 (defcustom helm-M-x-reverse-history nil
-  "The history source of `helm-M-x' appear in second position when non--nil."
+  "The history source of `helm-M-x' appear in second position when non-nil."
   :group 'helm-command
   :type 'boolean)
 
@@ -48,7 +48,9 @@
   :group 'helm-command
   :group 'helm-faces)
 
-(defface helm-M-x-key '((t (:foreground "orange" :underline t)))
+(defface helm-M-x-key
+  `((t ,@(and (>= emacs-major-version 27) '(:extend t))
+       :foreground "orange" :underline t))
   "Face used in helm-M-x to show keybinding."
   :group 'helm-command-faces)
 
@@ -56,7 +58,8 @@
 (defvar helm-M-x-input-history nil)
 (defvar helm-M-x-prefix-argument nil
   "Prefix argument before calling `helm-M-x'.")
-
+(defvar helm-M-x--timer nil)
+(defvar helm-M-x--unwind-forms-done nil)
 
 (defun helm-M-x-get-major-mode-command-alist (mode-map)
   "Return alist of MODE-MAP."
@@ -68,8 +71,8 @@
 
 (defun helm-get-mode-map-from-mode (mode)
   "Guess the mode-map name according to MODE.
-Some modes don't use conventional mode-map name
-so we need to guess mode-map name. e.g python-mode ==> py-mode-map.
+Some modes don't use conventional mode-map name so we need to
+guess mode-map name. E.g. `python-mode' ==> py-mode-map.
 Return nil if no mode-map found."
   (cl-loop ;; Start with a conventional mode-map name.
         with mode-map    = (intern-soft (format "%s-map" mode))
@@ -92,10 +95,12 @@ Return nil if no mode-map found."
 
 (defun helm-M-x-transformer-1 (candidates &optional sort)
   "Transformer function to show bindings in emacs commands.
-Show global bindings and local bindings according to current `major-mode'.
+Show global bindings and local bindings according to current
+`major-mode'.
 If SORT is non nil sort list with `helm-generic-sort-fn'.
 Note that SORT should not be used when fuzzy matching because
-fuzzy matching is running its own sort function with a different algorithm."
+fuzzy matching is running its own sort function with a different
+algorithm."
   (with-helm-current-buffer
     (cl-loop with local-map = (helm-M-x-current-mode-map-alist)
           for cand in candidates
@@ -196,18 +201,18 @@ fuzzy matching is running its own sort function with a different algorithm."
 (defun helm-M-x-read-extended-command (collection &optional predicate history)
   "Read or execute action on command name in COLLECTION or HISTORY.
 
-When `helm-M-x-use-completion-styles' is used, several actions as of
-`helm-type-command' are used and executed from here, otherwise this
-function returns the command as a symbol.
+When `helm-M-x-use-completion-styles' is used, several actions as
+of `helm-type-command' are used and executed from here, otherwise
+this function returns the command as a symbol.
 
-Helm completion is not provided when executing or defining kbd macros.
+Helm completion is not provided when executing or defining kbd
+macros.
 
-Arg COLLECTION should be an `obarray' but can be any object suitable
-for `try-completion'.  Arg PREDICATE is a function that default to
-`commandp' see also `try-completion'.
-Arg HISTORY default to `extended-command-history'."
+Arg COLLECTION should be an `obarray' but can be any object
+suitable for `try-completion'.  Arg PREDICATE is a function that
+default to `commandp' see also `try-completion'.  Arg HISTORY
+default to `extended-command-history'."
   (let* ((helm--mode-line-display-prefarg t)
-         (tm (run-at-time 1 0.1 'helm-M-x--notify-prefix-arg))
          (minibuffer-completion-confirm t)
          (pred (or predicate #'commandp))
          (metadata (unless (assq 'flex completion-styles-alist)
@@ -235,6 +240,7 @@ Arg HISTORY default to `extended-command-history'."
                           ((integerp helm-M-x-prefix-argument)
                            (format "%d " helm-M-x-prefix-argument)))
                          "M-x ")))
+    (setq helm-M-x--timer (run-at-time 1 0.1 'helm-M-x--notify-prefix-arg))
     ;; Fix Issue #2250, add `helm-move-selection-after-hook' which
     ;; reset prefix arg to nil only for this helm session.
     (add-hook 'helm-move-selection-after-hook
@@ -250,18 +256,36 @@ Arg HISTORY default to `extended-command-history'."
                 :prompt prompt
                 :buffer "*helm M-x*"
                 :history 'helm-M-x-input-history))
-      (cancel-timer tm)
-      (setq helm--mode-line-display-prefarg nil)
-      ;; Be sure to remove it here as well in case of quit.
-      (remove-hook 'helm-move-selection-after-hook
-                   'helm-M-x--move-selection-after-hook)
-      (remove-hook 'helm-before-action-hook
-                   'helm-M-x--before-action-hook))))
+      (helm-M-x--unwind-forms))))
+
+;; When running a command involving again helm from helm-M-x, the
+;; unwind-protect UNWINDS forms are executed only once this helm
+;; command exit leaving the helm-M-x timer running and other variables
+;; and hooks not unset, so the timer is now in a global var and all
+;; the forms that should normally run in unwind-protect are running as
+;; well as soon as helm-M-x-execute-command is called.
+(defun helm-M-x--unwind-forms (&optional done)
+  ;; helm-M-x--unwind-forms-done is non nil when it have been called
+  ;; once from helm-M-x-execute-command.
+  (unless helm-M-x--unwind-forms-done
+    (when (timerp helm-M-x--timer)
+      (cancel-timer helm-M-x--timer)
+      (setq helm-M-x--timer nil))
+    (setq helm--mode-line-display-prefarg nil)
+    ;; Be sure to remove it here as well in case of quit.
+    (remove-hook 'helm-move-selection-after-hook
+                 'helm-M-x--move-selection-after-hook)
+    (remove-hook 'helm-before-action-hook
+                 'helm-M-x--before-action-hook))
+  ;; Reset helm-M-x--unwind-forms-done to nil when DONE is
+  ;; unspecified.
+  (setq helm-M-x--unwind-forms-done done))
 
 (defun helm-M-x-execute-command (command)
   "Execute COMMAND as an editor command.
 COMMAND must be a symbol that satisfies the `commandp' predicate.
 Save COMMAND to `extended-command-history'."
+  (helm-M-x--unwind-forms t)
   (when command
     ;; Avoid having `this-command' set to *exit-minibuffer.
     (setq this-command command
@@ -271,18 +295,14 @@ Save COMMAND to `extended-command-history'."
     ;; use the value of arg otherwise use helm-current-prefix-arg.
     (let ((prefix-arg (or helm-current-prefix-arg helm-M-x-prefix-argument))
           (command-name (symbol-name command)))
-      (cl-flet ((save-hist
-                 (name)
-                 (setq extended-command-history
-                       (cons name (delete name extended-command-history)))))
-        (condition-case-unless-debug err
-            (progn
-              (command-execute command 'record)
-              (save-hist command-name))
-          (error
-           (when helm-M-x-always-save-history
-             (save-hist command-name))
-           (signal (car err) (cdr err))))))))
+      (condition-case-unless-debug err
+          (progn
+            (command-execute command 'record)
+            (add-to-history 'extended-command-history command-name))
+        (error
+         (when helm-M-x-always-save-history
+           (add-to-history 'extended-command-history command-name))
+         (signal (car err) (cdr err)))))))
 
 (defun helm-M-x--vanilla-M-x ()
   (helm-M-x-execute-command
@@ -298,12 +318,14 @@ Save COMMAND to `extended-command-history'."
 ;;;###autoload
 (defun helm-M-x (_arg)
   "Preconfigured `helm' for Emacs commands.
-It is `helm' replacement of regular `M-x' `execute-extended-command'.
+It is `helm' replacement of regular `M-x'
+`execute-extended-command'.
 
-Unlike regular `M-x' emacs vanilla `execute-extended-command' command,
-the prefix args if needed, can be passed AFTER starting `helm-M-x'.
-When a prefix arg is passed BEFORE starting `helm-M-x', the first `C-u'
-while in `helm-M-x' session will disable it.
+Unlike regular `M-x' Emacs vanilla `execute-extended-command'
+command, the prefix args if needed, can be passed AFTER starting
+`helm-M-x'.  When a prefix arg is passed BEFORE starting
+`helm-M-x', the first `C-u' while in `helm-M-x' session will
+disable it.
 
 You can get help on each command by persistent action."
   (interactive
