@@ -23,6 +23,8 @@
 (require 'helm-help)
 (require 'helm-utils)
 
+(declare-function 'helm-buffers-get-visible-buffers "helm-buffers.el")
+
 ;;; Internals
 ;;
 (defvar helm-source-occur nil)
@@ -118,12 +120,14 @@ Note that when using `buffer-substring' initialization will be slower."
 
 
 (defface helm-moccur-buffer
-    '((t (:foreground "DarkTurquoise" :underline t)))
+  `((t ,@(and (>= emacs-major-version 27) '(:extend t))
+       (:foreground "DarkTurquoise" :underline t)))
   "Face used to highlight occur buffer names."
   :group 'helm-occur)
 
 (defface helm-resume-need-update
-    '((t (:background "red")))
+  `((t ,@(and (>= emacs-major-version 27) '(:extend t))
+       (:background "red")))
   "Face used to flash occur buffer when it needs update."
   :group 'helm-occur)
 
@@ -143,7 +147,7 @@ buffers (i.e. a helm command using `helm-source-buffers-list' like
 `helm-mini') and use the multi occur buffers action.
 
 This is the helm implementation that collect lines matching pattern
-like vanilla emacs `occur' but have nothing to do with it, the search
+like vanilla Emacs `occur' but have nothing to do with it, the search
 engine beeing completely different and also much faster."
   (interactive)
   (setq helm-source-occur
@@ -178,8 +182,18 @@ engine beeing completely different and also much faster."
                  :truncate-lines helm-occur-truncate-lines)
         (deactivate-mark t)))))
 
+;;;###autoload
+(defun helm-occur-visible-buffers ()
+  "Run helm-occur on all visible buffers in frame."
+  (interactive)
+  (require 'helm-buffers)
+  (if (or (one-window-p) (region-active-p))
+      (call-interactively #'helm-occur)
+    (let ((buffers (helm-buffers-get-visible-buffers)))
+      (helm-multi-occur-1 (mapcar 'get-buffer buffers)))))
+
 (defun helm-occur-transformer (candidates source)
-  "Returns CANDIDATES prefixed with line number."
+  "Return CANDIDATES prefixed with line number."
   (cl-loop with buf = (helm-attr 'buffer-name source)
            for c in candidates collect
            (when (string-match helm-occur--search-buffer-regexp c)
@@ -200,7 +214,7 @@ engine beeing completely different and also much faster."
                    :initform nil)))
 
 (defun helm-occur-build-sources (buffers &optional source-name)
-  "Build sources for helm-occur for each buffer in BUFFERS list."
+  "Build sources for `helm-occur' for each buffer in BUFFERS list."
   (cl-loop for buf in buffers
            collect
            (helm-make-source (or source-name
@@ -254,7 +268,7 @@ engine beeing completely different and also much faster."
              :moccur-buffers buffers)))
 
 (defun helm-multi-occur-1 (buffers &optional input)
-  "Runs helm-occur on a list of buffers.
+  "Run `helm-occur' on a list of buffers.
 Each buffer's result is displayed in a separated source."
   (let* ((curbuf (current-buffer))
          (bufs (if helm-occur-always-search-in-current
@@ -287,7 +301,7 @@ Each buffer's result is displayed in a separated source."
 (cl-defun helm-occur-action (lineno
                                   &optional (method (quote buffer)))
   "Jump to line number LINENO with METHOD.
-arg METHOD can be one of buffer, buffer-other-window, buffer-other-frame."
+METHOD can be one of buffer, buffer-other-window, buffer-other-frame."
   (require 'helm-grep)
   (let ((buf (if (eq major-mode 'helm-occur-mode)
                  (get-text-property (point) 'buffer-name)
@@ -355,7 +369,7 @@ Same as `helm-occur-goto-line' but go in new frame."
 (put 'helm-moccur-run-save-buffer 'helm-only t)
 
 (defun helm-occur-right ()
-  "helm-occur action for right arrow.
+  "`helm-occur' action for right arrow.
 This is used when `helm-occur-use-ioccur-style-keys' is enabled.
 If follow is enabled (default) go to next source, otherwise execute
 persistent action."
@@ -494,7 +508,7 @@ persistent action."
   (helm-multi-occur-1 helm-occur--buffer-list helm-occur-mode--last-pattern))
 
 (defun helm-occur-buffer-substring-with-linums ()
-  "Returns current-buffer contents as a string with all lines
+  "Return current-buffer contents as a string with all lines
 numbered.  The property 'buffer-name is added to the whole string."
   (let ((bufstr (buffer-substring-no-properties (point-min) (point-max)))
         (bufname (buffer-name)))
@@ -582,7 +596,7 @@ numbered.  The property 'buffer-name is added to the whole string."
                   ":"
                   (propertize lineno 'face 'helm-grep-lineno)
                   ":"
-                  (helm-grep-highlight-match str t))
+                  (helm-grep-highlight-match str))
           candidate)))
 
 (define-derived-mode helm-occur-mode
