@@ -48,7 +48,7 @@
   "Advice for `cl--print-table' to make readable class slots docstrings."
   (let ((format "%s\n\n  Initform=%s\n\n%s"))
     (dolist (row rows)
-      (setcar row (propertize (car row) 'face 'italic))
+      (setcar row (propertize (car row) 'face 'bold))
       (setcdr row (nthcdr 1 (cdr row)))
       (insert "\n* " (apply #'format format row) "\n"))))
 
@@ -458,6 +458,13 @@
 
   Note that FUZZY-MATCH slot will overhide value of this slot.")
 
+   (match-on-real
+    :initarg :match-on-real
+    :initform nil
+    :custom boolean
+    :documentation
+    "  Match the real value of candidates when non nil.")
+
    (fuzzy-match
     :initarg :fuzzy-match
     :initform nil
@@ -652,11 +659,16 @@
     :initform nil
     :custom symbol
     :documentation
-    "  Prevent exiting with empty helm buffer.
-  For this to work `minibuffer-completion-confirm' must be let-bounded
-  around the helm call.
-  Same as `completing-read' require-match arg, possible values are `t'
-  or `confirm'.")
+    "  Same as `completing-read' require-match arg.
+  Possible values are:
+  - `t' which prevent exiting with an empty helm-buffer i.e. no matches.
+  - `confirm' which ask for confirmation i.e. need to press a second
+     time RET.
+  - `nil' is the default and is doing nothing i.e. returns nil when
+    pressing RET with an empty helm-buffer.
+  - Any other non nil values e.g. `ignore' allow exiting with
+    minibuffer contents as candidate value (in this case helm-buffer
+    is empty).")
 
    (group
     :initarg :group
@@ -906,7 +918,7 @@ Where OBJECT is an instance of an eieio class."
   (cl-loop for s in (object-slots object)
            for slot-val = (slot-value object s)
            when slot-val
-           collect (cons s (unless (eq t slot-val) slot-val))))
+           collect (cons s slot-val)))
 
 (defun helm-make-source (name class &rest args)
   "Build a `helm' source named NAME with ARGS for CLASS.
